@@ -133,7 +133,7 @@ xfs_bulkstat_single(int fd, xfs_ino_t *lastip, xfs_bstat_t *ubuffer)
 {
     xfs_fsop_bulkreq_t  bulkreq;
 
-    bulkreq.lastip = lastip;
+    bulkreq.lastip = (__u64 *)lastip;
     bulkreq.icount = 1;
     bulkreq.ubuffer = ubuffer;
     bulkreq.ocount = NULL;
@@ -146,7 +146,7 @@ xfs_bulkstat(int fd, xfs_ino_t *lastip, int icount,
 {
     xfs_fsop_bulkreq_t  bulkreq;
 
-    bulkreq.lastip = lastip;
+    bulkreq.lastip = (__u64 *)lastip;
     bulkreq.icount = icount;
     bulkreq.ubuffer = ubuffer;
     bulkreq.ocount = ocount;
@@ -1190,7 +1190,7 @@ packfile(char *fname, char *tname, int fd,
 			}
 			if (nfrags) {
 				/* Do a matching write to the tmp file */
-				wc = wc_b4;
+				wc_b4 = wc;
 				if (((wc = write(ffd, fbuf, wc)) != wc_b4)) {
 					fsrprintf(_("bad write of %d bytes "
 						"to %s: %s\n"),
@@ -1541,7 +1541,7 @@ tmp_init(char *mnt)
 	}
 	for (i=0; i < fsgeom.agcount; i++) {
 		sprintf(buf, "%s/.fsr/ag%d", mnt, i);
-		if (mkdir(buf, 0777) < 0) {
+		if (mkdir(buf, 0700) < 0) {
 			if (errno == EEXIST) {
 				if (dflag)
 					fsrprintf(
